/*
 wmtz.c
 Shows local time in different timezones + JD.
 Jan Lindblom <jlin@algonet.se> 1999-04-15

 wmtz.c was derived from:

   wminet.c
   Multi-function system monitor
   Dave Clark (clarkd@skynet.ca) (http://www.neotokyo.org/illusion)
   Martijn Pieterse (pieterse@xs4all.nl) (http://windowmaker.mezaway.org)
   and Antoine Nulle (warp@xs4all.nl) (http://windowmaker.mezaway.org)

 This software is licensed through the GNU General Public Licence.
 Read the COPYING file for details.

*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"

#include "wmtz.xpm"
char wmtz_mask_bits[64*64];
int  wmtz_mask_width = 64;
int  wmtz_mask_height = 64;


#define WMTZ_VERSION "0.5"
#define CHAR_WIDTH 6
#define STRSIZE 10

extern char *tzname[2];

typedef struct
{
    char label[10]; /* Time zone designation */
    int  diff;      /* Time zone diff. from UT in hours OR when label="JDN"...  */
} timezone_t;       /* ...numbers of days to subtract from JD */

timezone_t zone[6]; 
static struct tm *clk;
char uconfig_file[256];

void usage(void);
void printversion(void);
void BlitString(char *name, int x, int y);
void wmtz_routine(int, char **);
int ReadConfigInt(FILE *fp, char *setting, int *value);
int ReadConfigString(FILE *fp, char *setting, char *value);
int Read_Config_File( char *filename );

char *month[12];
char *week_day[7];

int main(int argc, char *argv[]) {

  int		i;

  uconfig_file[0] = 0;
  
  /* Parse Command Line */	
  for (i=1; i<argc; i++) {
    char *arg = argv[i];
    
    if (*arg=='-') {
      switch (arg[1]) {
      case 'd' :
	if (strcmp(arg+1, "display")) {
	  usage();
	  exit(1);
	}
	break;
      case 'g' :
	if (strcmp(arg+1, "geometry")) {
	  usage();
	  exit(1);
	}
	break;
      case 'v' :
	printversion();
	exit(0);
	break;
      case 'c' :
	if (argc > (i+1))
	  {
	    strcpy(uconfig_file, argv[i+1]);
	    i++;
	  }
	break;
      default:
	usage();
	exit(0);
	break;
      }
    }
  }

  month[0] = "JAN";  month[1] = "FEB";  month[2] = "MAR";
  month[3] = "APR";  month[4] = "MAY";  month[5] = "JUN";
  month[6] = "JUL";  month[7] = "AUG";  month[8] = "SEP";
  month[9] = "OUT";  month[10] = "NOV";  month[11] = "DEC";
  
  week_day[0] = "SUNDAY   ";
  week_day[1] = "MONDAY   ";
  week_day[2] = "TUESDAY  ";
  week_day[3] = "WEDNESDAY";
  week_day[4] = "THURSDAY ";
  week_day[5] = "FRIDAY   ";
  week_day[6] = "SATURDAY ";
  
  wmtz_routine(argc, argv);

  return 0;
}


/*******************************************************************************\
|* wmtz_routine								       *|
\*******************************************************************************/
void wmtz_routine(int argc, char **argv)
{
    int			j=0,k=0, hour=0;
    double              jd;
    float               swatch_beats;
    time_t		curtime;
    time_t		prevtime;
    char config_file[512];
    char buf[64];
    char blitstr[STRSIZE];

    createXBMfromXPM(wmtz_mask_bits, wmtz_master_xpm, wmtz_mask_width, wmtz_mask_height);
    
    openXwindow(argc, argv, wmtz_master_xpm, wmtz_mask_bits, wmtz_mask_width, wmtz_mask_height);

    memset(&zone, 0, sizeof(zone));

    // Read config file 
    if (uconfig_file[0] != 0)
    {
        // user-specified config file
        fprintf(stderr, "Using user-specified config file '%s'.\n", uconfig_file);
        Read_Config_File(uconfig_file);
    }
    else
    {
        snprintf(config_file,512, "%s/.wmtzrc", getenv("HOME"));

        if (!Read_Config_File(config_file))
        {
            // Fall back to /etc/wmtzrc
            snprintf(config_file,512, "/etc/wmtzrc");
        
            Read_Config_File(config_file);
        }
    }

    RedrawWindow();
    prevtime = time(0) - 1;
    
    
    while (1)
    {
        curtime = time(0);
        if ( curtime > prevtime)
        {
	  prevtime = curtime;
          /* Update the display */
          for (j=1; j<6; j++)
          {
            if (strncmp( zone[j].label, "xxx", 3) == 0 )
	    {
              BlitString("        ", 5, (11*(j-1)) + 5); 
	    }
	    /* Display local day/mon/year */
            else if (strncmp( zone[j].label, "DATE", 4) == 0 )
	    {
	      clk = localtime(&curtime);
	      while(clk->tm_year>99) clk->tm_year-=100;
	      snprintf(blitstr, STRSIZE,"%s %02d.%02d",month[clk->tm_mon],clk->tm_mday,clk->tm_year);
              BlitString(blitstr, 5, (11*(j-1)) + 5);  
	    }
	    /* Display local weekday */
            else if (strncmp( zone[j].label, "WDAY", 4) == 0 )
	    {
	      clk = localtime(&curtime);
	      snprintf(blitstr, STRSIZE,"%s",week_day[clk->tm_wday]);
	      BlitString(blitstr, 4, (11*(j-1)) + 5);  
	    }
           /* Display Internet time */
            else if (strncmp( zone[j].label,"BEATS", 5) == 0 )
            {
              /* Calculate Internet time */
              swatch_beats = (((curtime+3600)%86400)/86.4);
              snprintf (blitstr, STRSIZE, "BEATS;%3i", (int)swatch_beats);
              BlitString (blitstr, 5, (11*(j-1)) + 5);
            }
	    /* Display more precise internet time */
            else if (strncmp( zone[j].label,"@", 1) == 0 )
            {
              /* Calculate Internet time */
              swatch_beats = (float)(((curtime+3600)%86400)/86.4);
              snprintf (blitstr, STRSIZE, "@:%7.3f", swatch_beats);
              BlitString (blitstr, 5, (11*(j-1)) + 5);
            }
            else if (strncmp( zone[j].label,"Y2K", 3) == 0 )
	    { 
	      clk = gmtime(&curtime);
              /* Calculate number of days until 2000.0 */
	      jd = curtime/86400.0 - 10957.0 - (zone[j].diff * .04167);

              if (jd > 0)
	      {
                /* Print warning */
                BlitString ("TOO LATE", 5, (11*(j-1)) + 5);
	      }
              else
	      {
		jd*=-1;
                /* Print label */
		snprintf(blitstr, STRSIZE, "Y2K:%3.2f", jd );
		BlitString(blitstr, 5, (11*(j-1)) + 5);
	      }
	    }
            else if (strncmp( zone[j].label, "JDN", 3) == 0 )
	    {
 	      clk = gmtime(&curtime);
	      /* Calculate Julin Day Number */
	      jd = curtime/86400.0 + 2440587.5 - zone[j].diff;
	      snprintf(blitstr, STRSIZE, "%10f", jd );
	      BlitString(blitstr, 5, (11*(j-1)) + 5);

	    }
	    else if (strncmp( zone[j].label, "LOCAL", 5) == 0 )
	    {
	      clk = localtime(&curtime);
	      strncpy(buf, tzname[0], 3);
	      
              for (k=0; k<3; k++)
                if (buf[k] == 0)
                  buf[k] = ' ';

              buf[3] = ':';
              buf[4] = 0;
	      hour = clk->tm_hour;
              if (hour > 23 )
                hour -= 24;
              else if (hour < 0 )
                hour += 24;

	      // Print Label
              snprintf(blitstr, STRSIZE, "%s%02i.%02i",buf,hour,clk->tm_min);
              BlitString(blitstr, 5, (11*(j-1)) + 5);
	    }
            else 
            {
 	      clk = gmtime(&curtime);
              strncpy(buf, zone[j].label, 3);
                       
              for (k=0; k<3; k++)
                if (buf[k] == 0)
                  buf[k] = ' ';

              buf[3] = ':';
              buf[4] = 0;

              hour = clk->tm_hour - zone[j].diff;
              if (hour > 23 ) 
		hour -= 24;
              else if (hour < 0 ) 
		hour += 24;

	      // Print Label
              snprintf(blitstr, STRSIZE, "%s%02i.%02i",buf,hour,clk->tm_min);
              BlitString(blitstr, 5, (11*(j-1)) + 5);
	    }
          }                                     
        }
        RedrawWindow();
        usleep(100000);
    }        
}


// ReadConfigSetting
int ReadConfigString(FILE *fp, char *setting, char *value)
{
    char str[1024];
    char buf[1024];
    int i;
    int len;
    int slen;
    char *p=NULL;


    if (!fp)
    {
        return 0;
    }

    snprintf(str,1024, "%s=", setting);
    slen = strlen(str);
    
    fseek(fp, 0, SEEK_SET);

    while ( !feof(fp) )
    {
        
        if (!fgets(buf, 512, fp))
	  break;
        
        len = strlen(buf);

        // strip linefeed
        for (i=0; i!=len; i++)
	  if (buf[i] == '\n')
	    buf[i] = 0;


        //printf("Scanning '%s'...\n", buf);
        if ( strncmp(buf, str, strlen(str)) == 0)
        {
            // found our setting
	  
	  for(i=0; i!=slen; i++)
	    if ( buf[i] == '=' )
	      {
		p=buf+i+1;
		strcpy(value, p);
		return 1;
	      }
        }
    }    
    return 0;
}

int ReadConfigInt(FILE *fp, char *setting, int *value)
{
    char buf[1024];

    if (ReadConfigString(fp, setting, (char *) &buf))
    {
        *value = atoi(buf);
        return 1;
    }

    return 0;
}

int Read_Config_File( char *filename )
{
    FILE *fp;

    fp = fopen(filename, "r");
    if (fp)
    {
        ReadConfigString(fp, "time1", zone[1].label);
        ReadConfigString(fp, "time2", zone[2].label);
        ReadConfigString(fp, "time3", zone[3].label);
        ReadConfigString(fp, "time4", zone[4].label);
        ReadConfigString(fp, "time5", zone[5].label);
        ReadConfigInt(fp, "utdiff1", &zone[1].diff);
        ReadConfigInt(fp, "utdiff2", &zone[2].diff);
        ReadConfigInt(fp, "utdiff3", &zone[3].diff);
        ReadConfigInt(fp, "utdiff4", &zone[4].diff);
        ReadConfigInt(fp, "utdiff5", &zone[5].diff);

        fclose(fp);
        return 1;
    }
    else
    {
        perror("Read_Config_File");
        fprintf(stderr, "Unable to open %s, no settings read.\n", filename);
        return 0;
    }

}

// Blits a string at given co-ordinates
void BlitString(char *name, int x, int y)
{
    int		i;
    int		c;
    int		k;

    k = x;
    for (i=0; name[i]; i++)
    {

        c = toupper(name[i]); 
	if (c >= 'A' && c <= 'Z')
	{   /* its a letter */
	  c -= 'A';
	  copyXPMArea(c * CHAR_WIDTH, 74, CHAR_WIDTH, 8, k, y);
	  k += CHAR_WIDTH;
        }
        else if ( c >= '0' && c <= ':')
        {   
	  c -= '0';
	  copyXPMArea(c * CHAR_WIDTH, 64, CHAR_WIDTH, 8, k, y);
	  k += CHAR_WIDTH;
	}
	else if (c == ';') /* used as a slim ':' */
	{
	  copyXPMArea(60, 64, CHAR_WIDTH, 8, k, y);
	  k += 4;
	}
	else if (c=='.')
	  { 
	  copyXPMArea(115, 64, 4, 8, k, y);
	  k += 4;
	}
	else if (c=='@')
	{
	  copyXPMArea(108, 64, CHAR_WIDTH, 8, k, y);
	  k += CHAR_WIDTH;
	}
	else /* print a ' ' */
	{  
	  copyXPMArea(120, 64, CHAR_WIDTH, 8, k, y);
	  k += CHAR_WIDTH;
	}

    }
}



/*******************************************************************************\
|* usage								       *|
\*******************************************************************************/
void usage(void)
{
    fprintf(stderr, "\nwmtz - shows local time around the world and JD.\n");
    fprintf(stderr, "See ~/.wmtzrc or /etc/wmtzrc for configuration.\n\n");
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "    -display <display name>\n");
    fprintf(stderr, "    -geometry +XPOS+YPOS      initial window position\n");
    fprintf(stderr, "    -c <filename>             use specified config file\n");
    fprintf(stderr, "    -h                        this help screen\n");
    fprintf(stderr, "    -v                        print the version number\n");
    fprintf(stderr, "\n");
}



/*******************************************************************************\
|* printversion								       *|
\*******************************************************************************/
void printversion(void)
{
	fprintf(stderr, "wmtz v%s\n", WMTZ_VERSION);
}

